#include <iostream>
#include <thread>
#include <vector>
#include <barrier> // <==================


/// biathlon : langlauf + shoot
/// only top 3 of langlauf are still allowed to shoot

/// for demo purposes we will already at constructor time decide who is in top 3 ;-)

/// in the callback we print out that a round has finished, other interesting things,
/// might be resetting things for the next spin for example

/// callback must be noexcept

namespace
{
/// don't do this at home, global variables
const auto nrRunners{6};

std::barrier round2{nrRunners,
        [round = 0]() mutable noexcept {std::cout << " round " << ++round << " finished\n"; }
};

class Contender
{
public:
    Contender(int id, bool shootAllowed) :
        mId{id},
        mShootAllowed{shootAllowed}
    {}

    void operator()()
    {
        std::cout << mId << " skiing \n";
        std::this_thread::sleep_for(std::chrono::seconds(1 + mId * 1));
        if(mShootAllowed)
        {
            round2.arrive_and_wait();
        }
        else
        {
            round2.arrive_and_drop();
            return;
        }
        std::cout << mId << " shooting \n";
        std::this_thread::sleep_for(std::chrono::seconds(1 + mId * 1));

        round2.arrive_and_wait();
    }

private:
    int mId;
    bool mShootAllowed;
};

} // namespace


int main()
{
    std::vector<std::jthread> runnings;
    for(auto runner{0}; runner < nrRunners; ++runner)
    {
        runnings.emplace_back(Contender(runner, runner % 2));
    }

    std::this_thread::sleep_for(std::chrono::seconds(1));

    return 0;
}
