#include <iostream>
#include <thread>
#include <vector>
#include <latch>  // <==================

/// Running contest
///     runners prepare and get to the start line
///     run the race
///     wait till all runners finished and then hit the showers

///  some methods
/// countdown
/// wait
/// arrive_and_wait


/// 1 latch ==> one time sync point; can only be used ONCE

namespace
{
/// don't do this at home, global variables
const auto nrRunners{3};
std::latch atStartLine{nrRunners};
std::latch arrived{nrRunners};

void running(int runner)
{
    std::cout << runner << " Preparing\n";
    atStartLine.arrive_and_wait();  /// all threads/runners arrive here and wait till everyone is here; and then the thread can move on

    std::cout << runner << " Running\n";
    std::this_thread::sleep_for(std::chrono::seconds(2 + 3 * runner));
    std::cout << runner << " reached the finish line\n";

    arrived.count_down();
}

} // namespace


int main()
{
    std::vector<std::jthread> runnings;
    for(auto runner{0}; runner < nrRunners; ++runner)
    {
        runnings.emplace_back(running, runner);
        /// let's exaggerate the creation/launching part
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

    std::this_thread::sleep_for(std::chrono::seconds(1));

    std::cout << "waiting till all runners arrive at the finish line\n";
    arrived.wait();

    std::cout << "let's all hit the showers\n";
    return 0;
}
