#include <iostream>
#include <thread>
#include <vector>
#include <semaphore> // <==================
#include <syncstream> // <==================


namespace
{
const auto maxPlayers{16};
std::counting_semaphore<maxPlayers> sema{0};  /// at 0 --> so no players allowed yet

class Player
{
public:
    Player(int id) :
        mId{id}
    {}

    void operator()(std::stop_token stoken)
    {
        std::osyncstream sout{std::cout};   /// <===== synchronize cout
        while(!stoken.stop_requested())
        {
            //sout << mId << " waiting to join the game \n" << std::flush_emit;

            if(sema.try_acquire_for(std::chrono::milliseconds(200)))
            {
                sout << mId << " in the game \n" << std::flush_emit;
                std::this_thread::sleep_for(std::chrono::seconds(1));
                sout << mId << " stepping out of the game \n" << std::flush_emit;
                sema.release();
            }
        }
        sout << mId << " stopping\n" << std::flush_emit;
    }

private:
    int mId;
};

} // namespace


int main()
{
    std::stop_source ssource;
    auto token{ssource.get_token()};
    std::vector<std::jthread> threads;
    for(auto player{0}; player < maxPlayers; ++player)
    {
        threads.emplace_back(Player{player}, token);
    }

    std::this_thread::sleep_for(std::chrono::seconds(3));

    sema.release(2); /// 2 gamers allowed

    std::this_thread::sleep_for(std::chrono::seconds(5));
    sema.release(1); /// 1 more gamers allowed

    std::this_thread::sleep_for(std::chrono::seconds(5));

    std::osyncstream sout{std::cout};   /// <===== synchronize cout
    sout << "main says, time to go home\n\n\n\n" << std::flush_emit;
    ssource.request_stop();

    return 0;
}
