#include <iostream>
#include <thread>
#include <future>
#include <stop_token>

/// the constructor registers a callable for a token
/// the destructor unregisters it


/// multiple can be registered on the same stop token, order of being called is not defined


namespace
{

void doSomething(std::stop_token stoken, int instance)
{
    std::stop_callback cb{stoken, [instance](){
        std::cout << "callback for " << instance << '\n';
        } };


    while(!stoken.stop_requested())
    {
        std::cout << instance << " sleepy sleepy\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }
    std::cout << instance << " done\n";
}

} // namespace


int main()
{
    std::stop_source ssource;
    auto token{ssource.get_token()};

    std::jthread t1{doSomething, token, 1};
    std::jthread t2{doSomething, token, 2};
    std::thread t3{doSomething, token, 3};

    auto fut = std::async(std::launch::async, doSomething, token, 4);

    std::stop_callback cb{token, [](){
        std::cout << "callback for MAIN \n";
        } };


    /// we do some stuff, and suddenly we decide the threadS should stop
    std::this_thread::sleep_for(std::chrono::seconds(5));
    ssource.request_stop();     /// we request the source to stop

    t3.join();

    return 0;
}
