#include <iostream>
#include <thread>
#include <stop_token>

/// in which thread is the callback called :
///         if the stop was already requested before registration --> thread constructing the callback
///         otherwise in the thread invoking request_stop

namespace
{

void tooLate(std::stop_token stoken)
{
    std::cout << "tooLate : " << std::this_thread::get_id() << '\n';
    std::this_thread::sleep_for(std::chrono::seconds(4));

    std::stop_callback cb{stoken, [](){
        std::cout << "callback for tooLate " << std::this_thread::get_id() << '\n';
        } };
    std::cout << "tooLate has just registered the callback\n";

    while(!stoken.stop_requested())
    {
        std::cout << "tooLate sleepy sleepy\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }
    std::cout << "tooLate done\n";
}

void inTime(std::stop_token stoken)
{
    std::cout << "inTime : " << std::this_thread::get_id() << '\n';

    std::stop_callback cb{stoken, [](){
        std::cout << "callback for inTime " << std::this_thread::get_id() << '\n';
        } };
    std::cout << "inTime has just registered the callback\n";


    while(!stoken.stop_requested())
    {
        std::cout << "inTime sleepy sleepy\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }
    std::cout << "inTime done\n";
}
} // namespace

int main()
{
    std::cout << "main : " << std::this_thread::get_id() << '\n';

    std::stop_source ssource;
    auto token{ssource.get_token()};

    std::jthread t1{tooLate, token};
    std::jthread t2{inTime, token};

    std::this_thread::sleep_for(std::chrono::seconds(2));
    std::cout << "main requests to stop \n\n";
    ssource.request_stop();     /// we request the source to stop

    return 0;
}
