#include <iostream>
#include <thread>
#include <stop_token>

/// CO-OPERATIVE stopping
///     the party to be stopped, should check the token if stop is requested, and stop when it is

/// the jthread has its own stop source, and we can retrieve its associated token
/// on which we can request the stop, or we request the stop on the jthread

namespace
{

void doSomething(std::stop_token stoken)
{
    while(!stoken.stop_requested())
    {
        std::cout << "sleepy sleepy\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }
}

} // namespace


int main()
{
	{
		std::jthread t{doSomething};

		/// we do some stuff, and suddenly we decide the thread should stop
		std::this_thread::sleep_for(std::chrono::seconds(5));
		t.request_stop();

		std::cout << "and we AUTO joined with the thread we stopped\n";
	}

	std::cout << "new one : have it stop by going out of scope  (no more OOPS)\n";

	std::jthread t{doSomething};
	std::this_thread::sleep_for(std::chrono::seconds(3));

    return 0;
}
