#include <iostream>
#include <thread>
#include <future>

/// let's create our own stop source and use it for multiple threads
///     on 2 jthreads; and on one old style thread, and an async

namespace
{

void doSomething(std::stop_token stoken, int instance)
{
    while(!stoken.stop_requested())
    {
        std::cout << instance << " sleepy sleepy\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }
    std::cout << instance << " done\n";
}

} // namespace


int main()
{
    std::stop_source ssource;
    auto token{ssource.get_token()};

    std::jthread t1{doSomething, token, 1}; /// the internal one is not used, since we pass an explicit one in (we provide the parameters for the callable)
    std::jthread t2{doSomething, token, 2};
    std::thread t3{doSomething, token, 3};

    auto fut = std::async(std::launch::async, doSomething, token, 4);


    /// we do some stuff, and suddenly we decide the threadS should stop
    std::this_thread::sleep_for(std::chrono::seconds(5));
    std::cout << "Main is asking the threads to stop\n";
    ssource.request_stop();     /// we request the source to stop

    std::cout << "Main still has some work to do, while the threads have already been asked to stop.\n";
    std::this_thread::sleep_for(std::chrono::seconds(4));
    std::cout << "\n\nMain also done\n";

    t3.join();

    return 0;
}
